'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.default = neighborsIteration;

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _helpers = require('../helpers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; } /**
                                                                                                                                                                                                                   * Graphology Edges Iteration Specs
                                                                                                                                                                                                                   * =================================
                                                                                                                                                                                                                   *
                                                                                                                                                                                                                   * Testing the edges iteration-related methods of the graph.
                                                                                                                                                                                                                   */


var METHODS = ['neighbors', 'inNeighbors', 'outNeighbors', 'inboundNeighbors', 'outboundNeighbors', 'directedNeighbors', 'undirectedNeighbors'];

function neighborsIteration(Graph, checkers) {
  var invalid = checkers.invalid;
  var notFound = checkers.notFound;


  var graph = new Graph(null, { multi: true });

  graph.addNodesFrom(['John', 'Thomas', 'Martha', 'Roger', 'Catherine', 'Alone', 'Forever']);

  graph.addDirectedEdgeWithKey('J->T', 'John', 'Thomas');
  graph.addDirectedEdgeWithKey('J->M', 'John', 'Martha');
  graph.addDirectedEdgeWithKey('C->J', 'Catherine', 'John');

  graph.addUndirectedEdgeWithKey('M<->R', 'Martha', 'Roger');
  graph.addUndirectedEdgeWithKey('M<->J', 'Martha', 'John');
  graph.addUndirectedEdgeWithKey('J<->R', 'John', 'Roger');
  graph.addUndirectedEdgeWithKey('T<->M', 'Thomas', 'Martha');

  var TEST_DATA = {
    neighbors: {
      are: [['John', 'Martha', true], ['Martha', 'Catherine', false]],
      node: {
        key: 'John',
        neighbors: ['Catherine', 'Thomas', 'Martha', 'Roger']
      }
    },
    inNeighbors: {
      are: [['John', 'Martha', false], ['John', 'Roger', false], ['Martha', 'Catherine', false], ['Thomas', 'John', true]],
      node: {
        key: 'John',
        neighbors: ['Catherine']
      }
    },
    outNeighbors: {
      are: [['John', 'Martha', true], ['John', 'Roger', false], ['Martha', 'Catherine', false]],
      node: {
        key: 'John',
        neighbors: ['Thomas', 'Martha']
      }
    },
    inboundNeighbors: {
      are: [['John', 'Martha', true], ['John', 'Roger', false], ['Martha', 'Catherine', false]],
      node: {
        key: 'John',
        neighbors: ['Catherine', 'Martha']
      }
    },
    outboundNeighbors: {
      are: [['John', 'Martha', true], ['John', 'Roger', true], ['Martha', 'Catherine', false]],
      node: {
        key: 'John',
        neighbors: ['Thomas', 'Martha', 'Roger']
      }
    },
    directedNeighbors: {
      are: [['John', 'Martha', true], ['John', 'Roger', false], ['Martha', 'Catherine', false]],
      node: {
        key: 'John',
        neighbors: ['Catherine', 'Thomas', 'Martha']
      }
    },
    undirectedNeighbors: {
      are: [['John', 'Martha', true], ['John', 'Roger', true], ['Martha', 'Catherine', false]],
      node: {
        key: 'John',
        neighbors: ['Martha', 'Roger']
      }
    }
  };

  function commonTests(name) {
    return _defineProperty({}, '#.' + name, {
      'it should throw if too many arguments are provided.': function itShouldThrowIfTooManyArgumentsAreProvided() {
        _assert2.default.throws(function () {
          graph[name](1, 2, 3);
        }, invalid());
      },

      'it should throw if too few arguments are provided.': function itShouldThrowIfTooFewArgumentsAreProvided() {
        _assert2.default.throws(function () {
          graph[name]();
        }, invalid());
      },

      'it should throw when the node is not found.': function itShouldThrowWhenTheNodeIsNotFound() {
        _assert2.default.throws(function () {
          graph[name]('Test');
        }, notFound());

        if (~name.indexOf('count')) return;

        _assert2.default.throws(function () {
          graph[name]('Test', 'SecondTest');
        }, notFound());

        _assert2.default.throws(function () {
          graph[name]('Forever', 'Test');
        }, notFound());
      },

      'it should throw if any of the provided bunch node is not found.': function itShouldThrowIfAnyOfTheProvidedBunchNodeIsNotFound() {
        _assert2.default.throws(function () {
          graph[name](['Test']);
        }, notFound());
      }
    });
  }

  function specificTests(name, data) {
    var _ref4;

    var counterName = 'count' + (0, _helpers.capitalize)(name);

    return _ref4 = {}, _defineProperty(_ref4, '#.' + name, {
      'it should correctly return whether two nodes are neighbors.': function itShouldCorrectlyReturnWhetherTwoNodesAreNeighbors() {
        data.are.forEach(function (_ref2) {
          var _ref3 = _slicedToArray(_ref2, 3);

          var node1 = _ref3[0];
          var node2 = _ref3[1];
          var expectation = _ref3[2];

          _assert2.default.strictEqual(graph[name](node1, node2), expectation, name + ': ' + node1 + ' / ' + node2);
          _assert2.default.strictEqual(graph[name]('Forever', 'Alone'), false);
        });
      },

      'it should return the correct neighbors array.': function itShouldReturnTheCorrectNeighborsArray() {
        var neighbors = graph[name](data.node.key);

        _assert2.default.deepEqual(neighbors, data.node.neighbors);
        _assert2.default.deepEqual(graph[name]('Alone'), []);
      },

      'it should return the correct neighbors array from the provided bunch.': function itShouldReturnTheCorrectNeighborsArrayFromTheProvidedBunch() {
        (0, _helpers.testBunches)([data.node.key], function (bunch) {
          var neighbors = graph[name](bunch);

          _assert2.default.deepEqual(neighbors, data.node.neighbors);
        });

        (0, _helpers.testBunches)(['Forever', 'Alone'], function (bunch) {
          _assert2.default.deepEqual(graph[name](bunch), []);
        });
      }
    }), _defineProperty(_ref4, '#.' + counterName, {
      'it should return the correct neighbors count.': function itShouldReturnTheCorrectNeighborsCount() {
        var neighbors = graph[counterName](data.node.key);

        _assert2.default.strictEqual(neighbors, data.node.neighbors.length);
        _assert2.default.strictEqual(graph[counterName]('Alone'), 0);
      },

      'it should return the correct number of neighbors from the provided bunch.': function itShouldReturnTheCorrectNumberOfNeighborsFromTheProvidedBunch() {
        (0, _helpers.testBunches)([data.node.key], function (bunch) {
          _assert2.default.strictEqual(graph[counterName](bunch), data.node.neighbors.length);
        });
      }
    }), _ref4;
  }

  var tests = {};

  // Common tests
  METHODS.forEach(function (name) {
    return (0, _helpers.deepMerge)(tests, commonTests(name));
  });
  METHODS.forEach(function (name) {
    return (0, _helpers.deepMerge)(tests, commonTests('count' + (0, _helpers.capitalize)(name)));
  });

  // Specific tests
  for (var name in TEST_DATA) {
    (0, _helpers.deepMerge)(tests, specificTests(name, TEST_DATA[name]));
  }return tests;
}